searchData={"items":[{"type":"module","title":"Skogsra","doc":"This module defines the macros needed to use `Skogsra` e.g:\n\n```elixir\ndefmodule MyApp.Settings do\n  use Skogsra\n\n  @envdoc \"My hostname\"\n  app_env :my_hostname, :myapp, :hostname,\n    default: \"localhost\"\nend\n```","ref":"Skogsra.html"},{"type":"macro","title":"Skogsra.__using__/1","doc":"Imports `app_env/3` and `app_env/4`. Additionally generates the function\n`template(`\nFor now is just equivalent to use `import Skogsra`.","ref":"Skogsra.html#__using__/1"},{"type":"macro","title":"Skogsra.app_env/4","doc":"Creates a function to retrieve specific environment/application variables\nvalues.\n\nThe function created is named `function_name` and will get the value\nassociated with an application called `app_name` and one or several\n`parameters` keys. Optionally, receives a list of `options`.\n\nAvailable options:\n\nOption          | Type                    | Default              | Description\n:-------------- | :---------------------- | :------------------- | :----------\n`default`       | `any`                   | `nil`                | Sets the Default value for the variable.\n`type`          | `Skogsra.Env.type()`    | `:binary`            | Sets the explicit type for the variable.\n`os_env`        | `binary`                | autogenerated        | Overrides automatically generated OS environment variable name.\n`binding_order` | `Skogra.Env.bindings()` | `[:system, :config]` | Sets the load order for variable binding.\n`binding_skip`  | `Skogra.Env.bindings()` | `[]`                 | Which variable bindings should be skipped.\n`required`      | `boolean`               | `false`              | Whether the variable is required or not.\n`cached`        | `boolean`               | `true`               | Whether the variable should be cached or not.\n`namespace`     | `module`                | `nil`                | Overrides any namespace.\n\ne.g:\n\nFor the following declaration:\n\n```\napp_env :db_password, :myapp, [:mydb, :password],\n  default: \"password\",\n```\n\nwill generate:\n\n- `db_password/0` and `db_password/1` for getting the variable's value\nwithout or with namespace respectively. It returns `:ok` and `:error`\ntuples.\n- `db_password!/0` and `db_password!/1` for getting the variable's value\nwithout or with namespace respectively. It fails on error.\n- `reload_db_password/0` and `reload_db_password/1` for reloading the\nvariable's value in the cache without or with namespace respectively.\n- `put_db_password/1` and `put_db_password/2` for settings a new value for\nthe variable directly to the cache without or with namespace respectively.\n\nA call to `db_password/0` will try to get a value:\n\n1. From the OS environment variable `$MYAPP_MYDB_PASSWORD` (can be overriden\n   by the option `os_env`).\n2. From the configuration file e.g:\n   ```\n   config :myapp,\n     mydb: [password: \"some password\"]\n   ```\n3. From the default value if it exists (In this case, it would return\n   `\"password\"`).\n\nA call to `db_password/1` with namespace `Test` will try to get a value:\n\n1. From the OS environment variable `$TEST_MYAPP_MYDB_PASSWORD`.\n2. From the configuration file e.g:\n   ```\n   config :myapp, Test,\n     mydb: [password: \"some test password\"]\n   ```\n3. From the OS environment variable `$MYAPP_MYDB_PASSWORDT`.\n4. From the configuraton file e.g:\n   ```\n   config :myapp,\n     mydb: [password: \"some password\"]\n   ```\n5. From the default value if it exists. In our example, `\"password\"`.","ref":"Skogsra.html#app_env/4"},{"type":"module","title":"Skogsra.Core","doc":"This module defines the core API for Skogsra.","ref":"Skogsra.Core.html"},{"type":"function","title":"Skogsra.Core.get_env/1","doc":"Gets the value of a given `env`.","ref":"Skogsra.Core.html#get_env/1"},{"type":"function","title":"Skogsra.Core.get_env!/1","doc":"Gets the value of a given `env`. Fails on error.","ref":"Skogsra.Core.html#get_env!/1"},{"type":"function","title":"Skogsra.Core.put_env/2","doc":"Puts a new value for an `env`.","ref":"Skogsra.Core.html#put_env/2"},{"type":"function","title":"Skogsra.Core.reload_env/1","doc":"Reloads an `env` variable.","ref":"Skogsra.Core.html#reload_env/1"},{"type":"module","title":"Skogsra.Env","doc":"This module defines a `Skogsra` environment variable.","ref":"Skogsra.Env.html"},{"type":"function","title":"Skogsra.Env.__struct__/0","doc":"Environment variable struct.","ref":"Skogsra.Env.html#__struct__/0"},{"type":"function","title":"Skogsra.Env.binding_order/1","doc":"Gets the binding order for a `Skogsra` environment variable.","ref":"Skogsra.Env.html#binding_order/1"},{"type":"function","title":"Skogsra.Env.cached?/1","doc":"Whether the `Skogsra` environment variable is cached or not.","ref":"Skogsra.Env.html#cached?/1"},{"type":"function","title":"Skogsra.Env.default/1","doc":"Gets the default value for a `Skogsra` environment variable.","ref":"Skogsra.Env.html#default/1"},{"type":"function","title":"Skogsra.Env.extra_options/1","doc":"Gets extra options.","ref":"Skogsra.Env.html#extra_options/1"},{"type":"function","title":"Skogsra.Env.new/4","doc":"Creates a new `Skogsra` environment variable.","ref":"Skogsra.Env.html#new/4"},{"type":"function","title":"Skogsra.Env.os_env/1","doc":"Gets the OS variable name for the `Skogsra` environment variable.","ref":"Skogsra.Env.html#os_env/1"},{"type":"function","title":"Skogsra.Env.required?/1","doc":"Whether the `Skogsra` environment variable is required or not.","ref":"Skogsra.Env.html#required?/1"},{"type":"function","title":"Skogsra.Env.type/1","doc":"Gets the type of the `Skogsra` environment variable.","ref":"Skogsra.Env.html#type/1"},{"type":"type","title":"Skogsra.Env.app_name/0","doc":"Application name.","ref":"Skogsra.Env.html#t:app_name/0"},{"type":"type","title":"Skogsra.Env.binding/0","doc":"Variable binding.","ref":"Skogsra.Env.html#t:binding/0"},{"type":"type","title":"Skogsra.Env.bindings/0","doc":"Variable binding list.","ref":"Skogsra.Env.html#t:bindings/0"},{"type":"type","title":"Skogsra.Env.key/0","doc":"Key.","ref":"Skogsra.Env.html#t:key/0"},{"type":"type","title":"Skogsra.Env.keys/0","doc":"List of keys that lead to the value of the variable.","ref":"Skogsra.Env.html#t:keys/0"},{"type":"type","title":"Skogsra.Env.namespace/0","doc":"Variable namespace.","ref":"Skogsra.Env.html#t:namespace/0"},{"type":"type","title":"Skogsra.Env.option/0","doc":"Environment variable options.\n- `binding_order` - Variable binding load order.\n- `binding_skip` - Skips loading a variable from the list of bindings.\n- `os_env` - The name of the OS environment variable.\n- `type` - Type to cast the OS environment variable value.\n- `namespace` - Default namespace for the variable.\n- `default` - Default value.\n- `required` - Whether the variable is required or not.\n- `cached` - Whether the variable is cached or not.","ref":"Skogsra.Env.html#t:option/0"},{"type":"type","title":"Skogsra.Env.options/0","doc":"Environment variable options:","ref":"Skogsra.Env.html#t:options/0"},{"type":"type","title":"Skogsra.Env.t/0","doc":"Skogsra environment variable.","ref":"Skogsra.Env.html#t:t/0"},{"type":"type","title":"Skogsra.Env.type/0","doc":"Types.","ref":"Skogsra.Env.html#t:type/0"},{"type":"module","title":"Skogsra.Cache","doc":"This module defines the helpers for the variable cache. The cache works with\npersistent terms.","ref":"Skogsra.Cache.html"},{"type":"function","title":"Skogsra.Cache.get_env/1","doc":"Gets a variable value `env` from the cache.","ref":"Skogsra.Cache.html#get_env/1"},{"type":"function","title":"Skogsra.Cache.put_env/2","doc":"Stores a variable `env` in a persistent term.","ref":"Skogsra.Cache.html#put_env/2"},{"type":"behaviour","title":"Skogsra.Type","doc":"This module defines the functions and behaviours for casting `Skogsra` types.","ref":"Skogsra.Type.html"},{"type":"macro","title":"Skogsra.Type.__using__/1","doc":"Uses `Skogsra.Type` for implementing the behaviour e.g. a naive implementation\nfor casting `\"1, 2, 3, 4\"` to `[\"1\", \"2\", \"3\", \"4\"]` would be:\n\n```\ndefmodule MyList do\n  use Skogsra.Type\n\n  def cast(value) when is_binary(value) do\n    list =\n      value\n      |> String.split(~r/,/)\n      |> Enum.map(&String.trim/1)\n    {:ok, list}\n  end\n\n  def cast(_) do\n    :error\n  end\nend\n```","ref":"Skogsra.Type.html#__using__/1"},{"type":"callback","title":"Skogsra.Type.cast/1","doc":"Callback for casting a value.","ref":"Skogsra.Type.html#c:cast/1"},{"type":"function","title":"Skogsra.Type.cast/2","doc":"Casts an environment variable.","ref":"Skogsra.Type.html#cast/2"},{"type":"module","title":"Skogsra.App","doc":"This module defines the functions to get variables from the application\nconfiguration.","ref":"Skogsra.App.html"},{"type":"function","title":"Skogsra.App.preload/1","doc":"Overrides an config value with Skogsra's value.","ref":"Skogsra.App.html#preload/1"},{"type":"behaviour","title":"Skogsra.Binding","doc":"Variable binding behaviour.","ref":"Skogsra.Binding.html"},{"type":"macro","title":"Skogsra.Binding.__using__/1","doc":"Uses the `Skogsra.Binding` behaviour.","ref":"Skogsra.Binding.html#__using__/1"},{"type":"function","title":"Skogsra.Binding.get_env/2","doc":"","ref":"Skogsra.Binding.html#get_env/2"},{"type":"callback","title":"Skogsra.Binding.get_env/2","doc":"Callback for getting an environment variable.","ref":"Skogsra.Binding.html#c:get_env/2"},{"type":"callback","title":"Skogsra.Binding.init/1","doc":"Callback for initializing binding.","ref":"Skogsra.Binding.html#c:init/1"},{"type":"module","title":"Skogsra.Sys","doc":"This module defines the functions to get environment variables from the OS.","ref":"Skogsra.Sys.html"},{"type":"module","title":"Skogsra.Docs","doc":"This module defines the documentation generators.","ref":"Skogsra.Docs.html"},{"type":"function","title":"Skogsra.Docs.gen_full_docs/6","doc":"Generates docs for a variable given its `module`, `function_name`,\n`app_name`, `keys` to find it, `options` and custom `docs`.","ref":"Skogsra.Docs.html#gen_full_docs/6"},{"type":"function","title":"Skogsra.Docs.gen_put_docs/3","doc":"Generates put docs for a variable given its `module` and `function_name`.","ref":"Skogsra.Docs.html#gen_put_docs/3"},{"type":"function","title":"Skogsra.Docs.gen_reload_docs/3","doc":"Generates reload docs for a variable given its `module` and `function_name`.","ref":"Skogsra.Docs.html#gen_reload_docs/3"},{"type":"function","title":"Skogsra.Docs.gen_short_docs/3","doc":"Generates short docs for a variable given its `module`, `function_name`,\nand custom `docs`.","ref":"Skogsra.Docs.html#gen_short_docs/3"},{"type":"type","title":"Skogsra.Docs.docs/0","doc":"Custom docs.","ref":"Skogsra.Docs.html#t:docs/0"},{"type":"type","title":"Skogsra.Docs.function_name/0","doc":"Function name.","ref":"Skogsra.Docs.html#t:function_name/0"},{"type":"module","title":"Skogsra.Spec","doc":"This module defines the spec generators.","ref":"Skogsra.Spec.html"},{"type":"function","title":"Skogsra.Spec.gen_bang_spec/2","doc":"Generates spec for a bang function.","ref":"Skogsra.Spec.html#gen_bang_spec/2"},{"type":"function","title":"Skogsra.Spec.gen_full_spec/2","doc":"Generates spec for a configuration function.","ref":"Skogsra.Spec.html#gen_full_spec/2"},{"type":"function","title":"Skogsra.Spec.gen_put_spec/2","doc":"Generates put spec for a configuration function.","ref":"Skogsra.Spec.html#gen_put_spec/2"},{"type":"function","title":"Skogsra.Spec.gen_reload_spec/2","doc":"Generates reload spec for a configuration function.","ref":"Skogsra.Spec.html#gen_reload_spec/2"},{"type":"module","title":"Skogsra.Template","doc":"This module defines several templates for OS environment variable definitions.","ref":"Skogsra.Template.html"},{"type":"function","title":"Skogsra.Template.__struct__/0","doc":"Template internal structure.","ref":"Skogsra.Template.html#__struct__/0"},{"type":"function","title":"Skogsra.Template.generate/2","doc":"Generates a template.","ref":"Skogsra.Template.html#generate/2"},{"type":"function","title":"Skogsra.Template.new/1","doc":"Builds a template struct from a map.","ref":"Skogsra.Template.html#new/1"},{"type":"type","title":"Skogsra.Template.t/0","doc":"A template struct.","ref":"Skogsra.Template.html#t:t/0"},{"type":"type","title":"Skogsra.Template.type/0","doc":"Template type.","ref":"Skogsra.Template.html#t:type/0"},{"type":"module","title":"Skogsra.Provider.Json","doc":"This module defines a JSON config provider for Skogsra.\n\n> **Important**: You need to add `{:jason, \"~> 1.1\"}` as a dependency\n> along with `Skogsra` because the dependency is optional.\n\nThe following is an example for `Ecto` configuration in a file called\n`/etc/my_app/config.yml`:\n\n```json\n[\n  {\n    \"app\": \"my_app\",\n    \"module\": \"MyApp.Repo\",\n    \"config\": {\n      \"database\": \"my_app_db\",\n      \"username\": \"postgres\",\n      \"password\": \"postgres\",\n      \"hostname\": \"localhost\",\n      \"port\": 5432\n    }\n  }\n]\n```\n\nThen in your release configuration you can add the following:\n\n```elixir\nconfig_providers: [{Skogsra.Provider.Json, [\"/etc/my_app/config.json\"]}]\n```\n\nOnce the system boots, it'll parse and add the JSON configuration.","ref":"Skogsra.Provider.Json.html"},{"type":"module","title":"Skogsra.Provider.Yaml","doc":"This module defines a YAML config provider for Skogsra.\n\n> **Important**: You need to add `{:yamerl, \"~> 0.7\"}` as a dependency\n> along with `Skogsra` because the dependency is optional.\n\nThe following is an example for `Ecto` configuration:\n\n```yaml\n# file: /etc/my_app/config.yml\n- app: \"my_app\"\n  module: \"MyApp.Repo\"\n  config:\n    database: \"my_app_db\"\n    username: \"postgres\"\n    password: \"postgres\"\n    hostname: \"localhost\"\n    port: 5432\n```\n\nThen in your release configuration you can add the following:\n\n```elixir\nconfig_providers: [{Skogsra.Provider.Yaml, [\"/etc/my_app/config.yml\"]}]\n```\n\nOnce the system boots, it'll parse and add the YAML configuration.","ref":"Skogsra.Provider.Yaml.html"},{"type":"type","title":"Skogsra.Provider.Yaml.key/0","doc":"","ref":"Skogsra.Provider.Yaml.html#t:key/0"},{"type":"extras","title":"Skogsrå","doc":"# Skogsrå\n\n[![Build Status](https://travis-ci.org/gmtprime/skogsra.svg?branch=master)](https://travis-ci.org/gmtprime/skogsra) [![Hex pm](http://img.shields.io/hexpm/v/skogsra.svg?style=flat)](https://hex.pm/packages/skogsra) [![hex.pm downloads](https://img.shields.io/hexpm/dt/skogsra.svg?style=flat)](https://hex.pm/packages/skogsra) [![Coverage Status](https://coveralls.io/repos/github/gmtprime/skogsra/badge.svg?branch=master)](https://coveralls.io/github/gmtprime/skogsra?branch=master)\n\n> The _Skogsrå_ was a mythical creature of the forest that appears in the form\n> of a small, beautiful woman with a seemingly friendly temperament. However,\n> those who are enticed into following her into the forest are never seen\n> again.\n\nThis library attempts to improve the use of OS environment variables for\napplication configuration:\n\n* Variable defaults.\n* Automatic type casting of values.\n* Automatic docs and spec generation.\n* OS environment template generation.\n* Posibility to override Elixir configuration.\n* Runtime reloading.\n* Setting variable's values at runtime.\n* Fast cached values access by using `:persistent_term` as temporal storage.\n* Custom variable bindings.\n* YAML and JSON configuration providers for Elixir releases.","ref":"readme.html"},{"type":"extras","title":"Small example - Skogsrå","doc":"You would create a config module e.g:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"My hostname\"\n  app_env :my_hostname, :myapp, :hostname,\n    default: \"localhost\"\nend\n```\n\nCalling `MyApp.Config.my_hostname()` will retrieve the value for the\nhostname in the following order:\n\n1. From the OS environment variable `$MYAPP_HOSTNAME` (can be overriden by the\n   option `os_env`).\n2. From the configuration file e.g:\n   ```elixir\n   config :myapp,\n     hostname: \"my.custom.host\"\n   ```\n3. From the default value if it exists (In this case, it would return\n   `\"localhost\"`).","ref":"readme.html#small-example"},{"type":"extras","title":"Available options - Skogsrå","doc":"There are several options for configuring an environment variables:\n\nOption          | Type                                                 | Default              | Description\n:-------------- | :--------------------------------------------------- | :------------------- | :----------\n`default`       | `any`                                                | `nil`                | Sets the Default value for the variable.\n`type`          | [`Skogsra.Env.type()`](#types)                       | `:any`               | Sets the explicit type for the variable.\n`os_env`        | `binary`                                             | autogenerated        | Overrides automatically generated OS environment variable name.\n`binding_order` | [`Skogra.Env.bindings()`](#custom-variable-bindings) | `[:system, :config]` | Sets the load order for variable binding.\n`binding_skip`  | [`Skogra.Env.bindings()`](#custom-variable-bindings) | `[]`                 | Which variable bindings should be skipped.\n`required`      | `boolean`                                            | `false`              | Whether the variable is required or not.\n`cached`        | `boolean`                                            | `true`               | Whether the variable should be cached or not.\n\n> **IMPORTANT**: Options `skip_system: true` and `skip_config: true` have been\n> deprecated in favour of `binding_skip: [:system]` and `binding_skip: [:config]`\n> respectively.\n\nAdditional topics:\n\n- [Types](#types)\n- [Automatic type casting](#automatic-type-casting).\n- [Explicit type casting](#explicit-type-casting).\n- [Custom types](#custom-types).\n- [Explicit OS environment variable names](#explicit-os-environment-variable-name).\n- [Required variables](#required-variables).\n- [Overriding Elixir Configuration](#overriding-elixir-configuration).\n- [Caching variables](#caching-variables).\n- [Handling different environments](#handling-different-environments).\n- [Setting and reloading variables](#setting-and-reloading-variables).\n- [Automatic docs generation](#automatic-docs-generation).\n- [Automatic spec generation](#automatic-spec-generation).\n- [Automatic template generation](#automatic-template.generation).\n- [Custom variable bindings](#custom-variable-bindings).\n- [Elixir release YAML and JSON config providers](#elixir-release-yaml-and-json-config-providers).\n- [Using with Hab](#using-with-hab).\n- [Installation](#installation).","ref":"readme.html#available-options"},{"type":"extras","title":"Types - Skogsrå","doc":"Skogsrå implements several built-in types. Some of them can be\n[inferred from the default value](#automatic-type-casting) while others must be\n[set explicitly](#explicit-type-casting):\n\nType                          | Can be inferred? | Description\n:---------------------------- | :--------------- | :---------------\n`:any`                        | Yes              | Any Elixir term.\n`:binary`                     | Yes              | A string e.g. `\"localhost\"`.\n`:integer`                    | Yes              | An integer e.g. `42`.\n`:neg_integer`                | No               | A negative integer.\n`:non_neg_integer`            | No               | A non negative integer.\n`:pos_integer`                | No               | A positive integer.\n`:float`                      | Yes              | A float e.g. `42.0`.\n`:boolean`                    | Yes              | A boolean e.g. `false`.\n`:atom`                       | Yes              | An atom e.g. `:ok`.\n`:module`                     | No               | An existent module e.g. `Enum`.\n`:unsafe_module`              | No               | A module e.g. `SomeModule`.\n`Skogsra.Type` implementation | No               | See [Custom types](#custom-types) section.","ref":"readme.html#types"},{"type":"extras","title":"Automatic type casting - Skogsrå","doc":"If the `default` value is set (and no explicit `type` is found), the variable\nvalue will be casted as the same type of the default value. For this to work,\nthe default value should be of the following types:\n\n- `:any`\n- `:binary`\n- `:integer`\n- `:float`\n- `:boolean`\n- `:atom`\n\ne.g. in the following example, the value will be casted to `:atom`\nautomatically:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"My environment\"\n  app_env :my_environment, :myapp, :environment,\n    default: :prod\nend\n```\n\nIf either of the system OS or the application environment variables are defined,\nSkogsrå will try to cast their values to the default value's type which it\n`atom` e.g:\n\n```elixir\niex(1)> System.get_env(\"MYAPP_ENVIRONMENT\")\n\"staging\"\niex(2)> MyApp.Config.my_environment()\n{:ok, :staging}\n```\n\nor\n\n```elixir\niex(1)> Application.get_env(:myapp, :environment)\n\"staging\"\niex(2)> MyApp.Config.my_environment()\n{:ok, :staging}\n```\n\n> **Note**: If the variable is already of the desired type, it won't be casted.","ref":"readme.html#automatic-type-casting"},{"type":"extras","title":"Explicit type casting - Skogsrå","doc":"A type can be explicitly set. The available types are the following:\n\n - `:any` (default).\n - `:binary`.\n - `:integer`.\n - `:neg_integer`.\n - `:non_neg_integer`.\n - `:pos_integer`.\n - `:float`.\n - `:boolean`.\n - `:atom`.\n - `:module` (modules loaded in the system).\n - `:unsafe_module` (modules that might or might not be loaded in the system)\n - A module name with an implementation for the behaviour `Skogsra.Type`.\n\ne.g. in the following example, the value will be casted to `:pos_integer`\naccording to the variable definition:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"Port\"\n  app_env :my_port, :myapp, :port,\n    default: 4000,\n    type: :pos_integer\nend\n```\n\nIf either of the system OS or the application environment variables are defined,\nSkogsrå will try to cast their values to a positive integer.\n\n```elixir\niex(1)> System.get_env(\"MYAPP_PORT\")\n\"42\"\niex(2)> MyApp.Config.my_port()\n{:ok, 42}\n```\n\nor\n\n```elixir\niex(1)> Application.get_env(:myapp, :port)\n42\niex(2)> MyApp.Config.my_port()\n{:ok, 42}\n```","ref":"readme.html#explicit-type-casting"},{"type":"extras","title":"Custom types - Skogsrå","doc":"Skogsrå's types can be extented by implementing `Skogra.Type` behaviour e.g. a\npossible implementation for casting `\"1, 2, 3, 4\"` to `[integer()]` would be:\n\n```elixir\ndefmodule MyList do\n  use Skogsra.Type\n\n  @impl Skogsra.Type\n  def cast(value)\n\n  def cast(value) when is_binary(value) do\n    list =\n      value\n      |> String.split(~r/,/)\n      |> Stream.map(&String.trim/1)\n      |> Enum.map(&String.to_integer/1)\n\n    {:ok, list}\n  end\n\n  def cast(value) when is_list(value) do\n    if Enum.all?(value, &is_integer/1), do: {:ok, value}, else: :error\n  end\n\n  def cast(_) do\n    :error\n  end\nend\n```\n\nIf then we define the following enviroment variable with Skogsrå:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  app_env :my_integers, :myapp, :integers,\n    type: MyList\nend\n```\n\nIf either of the system OS or the application environment variables are defined,\nSkogsrå will try to cast their values using our implementation e.g:\n\n```elixir\niex(1)> System.get_env(\"MYAPP_INTEGERS\")\n\"1, 2, 3\"\niex(2)> MyApp.Config.my_integers()\n{:ok, [1, 2, 3]}\n```\n\nor\n\n```elixir\niex(1)> Application.get_env(:myapp, :integers)\n[1, 2, 3]\niex(2)> MyApp.Config.my_integers()\n{:ok, [1, 2, 3]}\n```\n\n> **Important**: The `default` value is not cast according to `type`.","ref":"readme.html#custom-types"},{"type":"extras","title":"Explicit OS environment variable names - Skogsrå","doc":"Though Skogsrå automatically generates the names for the OS environment\nvariables, they can be overriden by using the option `os_env` e.g:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  app_env :db_hostname, :myapp, [:postgres, :hostname],\n    os_env: \"PGHOST\"\nend\n```\n\nThis will override the value `$MYAPP_POSTGRES_HOSTNAME` with `$PGHOST` e.g:\n\n```elixir\niex(1)> System.get_env(\"MYAPP_POSTGRES_HOSTNAME\")\n\"unreachable\"\niex(2)> System.get_env(\"PGHOST\")\n\"reachable\"\niex(3)> MyApp.Config.db_hostname()\n{:ok, \"reachable\"}\n```","ref":"readme.html#explicit-os-environment-variable-names"},{"type":"extras","title":"Required variables - Skogsrå","doc":"It is possible to set a environment variable as required with the `required`\noption e.g:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"My port\"\n  app_env :my_port, :myapp, :port,\n    required: true\nend\n```\n\nIf none of the system OS or the application environment variables are defined,\nSkogsrå will return an error e.g:\n\n```elixir\niex(1)> System.get_env(\"MYAPP_PORT\")\nnil\niex(2)> Application.get_env(:myapp, :port)\nnil\niex(3)> MyApp.Config.my_port()\n{:error, \"Variable port in app myapp is undefined\"}\n```\n\nThe module will also provide `validate` and `validate!` functions that can be\nused in your application startup phase to verify that *all* required variables\nare present e.g:\n\n```elixir\niex(1)> System.get_env(\"MYAPP_PORT\")\nnil\niex(2)> Application.get_env(:myapp, :port)\nnil\niex(3)> MyApp.Config.validate!()\n** (RuntimeError) Variable port in app myapp is undefined\n```","ref":"readme.html#required-variables"},{"type":"extras","title":"Handling different environments - Skogsrå","doc":"If it's necessary to keep several environments, it's possible to use a\n`namespace` e.g. given the following variable:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"My port\"\n  app_env :my_port, :myapp, :port,\n    default: 4000\nend\n```\n\nCalling `MyApp.Config.my_port(Test)` will retrieve the value for the hostname\nin the following order:\n\n1. From the OS environment variable `$TEST_MYAPP_PORT`.\n2. From the configuration file e.g:\n   ```elixir\n   config :myapp, Test,\n     port: 4001\n   ```\n3. From the OS environment variable `$MYAPP_PORT`.\n4. From the configuraton file e.g:\n   ```elixir\n   config :myapp,\n     port: 80\n   ```\n5. From the default value if it exists. In our example, `4000`.\n\nThe ability of loading different environments allows us to do the following\nwith our configuration file:\n\n```elixir\n# file: config/config.exs\nuse Mix.Config\n\nconfig :myapp, Prod\n  port: 80\n\nconfig :myapp, Test,\n  port: 4001\n\nconfig :myapp,\n  port: 4000\n```\n\nWhile our Skogsrå module would look like:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"Application environment\"\n  app_env :env, :myapp, :environment,\n    type: :unsafe_module\n\n  @envdoc \"Application port\"\n  app_env :port, :myapp, :port,\n    default: 4000\nend\n```\n\nThe we can retrieve the values depending on the value of the OS environment\nvariable `$MYAPP_ENVIRONMENT`:\n\n```elixir\n...\nwith {:ok, env} <- MyApp.Config.env(),\n     {:ok, port} <- Myapp.Config.port(env) do\n  ... do something with the port ...\nend\n...\n```","ref":"readme.html#handling-different-environments"},{"type":"extras","title":"Overriding Elixir Configuration - Skogsrå","doc":"Not all the libraries will use Skogsrå and, though is a shame, we can always\noverride a config value using the function `preload/0` e.g.\n\nLet's say we have a Phoenix application and we want to configure our endpoint\nwith the following variables.\n\n- The OS environment variable `PORT` for our app's HTTP port.\n- The OS environment variable `SECRET_KEY_BASE` for our secret key base.\n\nA regular Phoenix configuration will look like the following:\n\n```elixir\nuse Mix.Config\n\nsecret_key_base =\n  System.get_env(\"SECRET_KEY_BASE\") ||\n    raise \"\"\"\n    environment variable SECRET_KEY_BASE is missing.\n    You can generate one by calling: mix phx.gen.secret\n    \"\"\"\n\n\nconfig :myapp_web, MyappWeb.Endpoint,\n  http: [\n    port: String.to_integer(System.get_env(\"PORT\") || \"4000\"),\n    transport_options: [socket_opts: [:inet6]]\n  ],\n  secret_key_base: secret_key_base\n```\n\nWith Skogsrå, we can reduce the config to the following:\n\n```elixir\nuse Mix.Config\n\nconfig :myapp_web, Myapp.Endpoint,\n  http: [\n    transport_options: [socket_opts: [:inet6]]\n  ]\n```\n\nThen, handle our variables in our Skogsrå config module:\n\n```elixir\ndefmodule MyappWeb.Config do\n  use Skogsra\n\n  @envdoc \"\"\"\n  App port.\n  \"\"\"\n  app_env :port, :myapp, [:http, :port],\n    os_env: \"PORT\",\n    default: 4000\n\n  @envdoc \"\"\"\n  Secret key base.\n  \"\"\"\n  app_env :secret_key_base, :myapp, :secret_key_base,\n    os_env: \"SECRET_KEY_BASE\",\n    default: \"+/JwMGGtsTVOoX5gQrCMn8aHKfKDdUK8GeAKJ2fIUabUnmWTwg+zsCy4pAOmOdTs\"\nend\n```\n\nAnd finally calling the preload function in our application init:\n\n```elixir\ndefmodule MyappWeb.Application do\n  use Application\n\n  def start(_type, _args) do\n    MyappWeb.Config.preload(MyappWeb.Endpoint)\n\n    children = [\n      MyappWeb.Endpoint\n    ]\n\n    ...\n  end\nend\n```\n\nThe function `MyappWeb.Config.preload/1` will override any configuration for\n`MyappWeb.Endpoint` as long as is a runtime config.\n\nOur app now will try to get the port and secret key from the OS environment\nvariables first and it will fallback to the defaults.","ref":"readme.html#overriding-elixir-configuration"},{"type":"extras","title":"Caching variables - Skogsrå","doc":"By default, Skogsrå caches the values of the variables using\n`:persistent_term` Erlang module. This makes reads very fast, but **writes are\nvery slow**.\n\nSo avoid setting or reloading values to avoid performance issues (see\n[Setting and reloading variables](#setting-and-reloading-variables)).\n\nIf you don't want to cache the values, you can set it to `false`:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  app_env :value, :myapp, :value,\n    cached: false\nend\n```","ref":"readme.html#caching-variables"},{"type":"extras","title":"Setting and reloading variables - Skogsrå","doc":"Every variable definition will generate two additional functions for setting\nand reloading the values e.g:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"My port\"\n  app_env :my_port, :myapp, :port,\n    default: 4000\nend\n```\n\nwill have the functions:\n\n- `MyApp.Config.put_my_port/1` for setting the value of the variable at\n  runtime.\n- `MyApp.Config.reload_my_port/` for reloading the value of the variable at\n  runtime.","ref":"readme.html#setting-and-reloading-variables"},{"type":"extras","title":"Automatic docs generation - Skogsrå","doc":"It's possible to document a configuration variables using the module attribute\n`@envdoc` e.g:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"My port\"\n  app_env :my_port, :myapp, :port,\n    default: 4000\nend\n```\n\nSkogsra then will automatically generate instructions on how to use the\nvariable. This extra documentation can be disable with the following:\n\n```elixir\nconfig :skogsra,\n  generate_docs: false\n```\n\n> **Note**: for \"private\" configuration variables you can use `@envdoc false`.","ref":"readme.html#automatic-docs-generation"},{"type":"extras","title":"Automatic spec generation - Skogsrå","doc":"Skogsra will try to generate the appropriate spec for every function generated.\nIn our example, given the default value is an integer, the function spec will be\nthe following:\n\n```elixir\n@spec my_port() :: {:ok, integer()} | {:error, binary()}\n@spec my_port(Skogsra.Env.namespace()) ::\n        {:ok, integer()} | {:error, binary()}\n```\n\n> **Note**: The same applies for `my_port!/0`, `reload_my_port/0` and\n> `put_my_port/1`.\n\nSpecs are generated following the following table:\n\nAssuming the type is set to `:integer` explicitly (`type: :integer`) or\nimplicitly (`default: 4000`), the specs should follow the following table:\n\nRequired     | Has default     | Spec\n:----------: | :-------------: | :----\n`true`       | `true`          | `@spec my_port() :: {:ok, integer()} \\| {:error, binary()}`\n`true`       | `false`         | `@spec my_port() :: {:ok, integer()} \\| {:error, binary()}`\n`false`      | `true`          | `@spec my_port() :: {:ok, integer()} \\| {:error, binary()}`\n`false`      | `false`         | `@spec my_port() :: {:ok, nil \\| integer()} \\| {:error, binary()}`","ref":"readme.html#automatic-spec-generation"},{"type":"extras","title":"Automatic template generation - Skogsrå","doc":"Every Skogsra module includes the functions `template/1` and `template/2` for\ngenerating OS environment variable files e.g. continuing our example:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"My port\"\n  app_env :my_port, :myapp, :port,\n    default: 4000\nend\n```\n\n- For Elixir releases:\n\n  ```elixir\n  iex(1)> Myapp.Config.template(\"env\")\n  :ok\n  ```\n\n  will generate the file `env` with the following contents:\n\n  ```bash\n  # DOCS My port\n  # TYPE integer\n  MYAPP_PORT=\"Elixir.Application\"\n  ```\n- For Unix:\n\n  ```elixir\n  iex(1)> Myapp.Config.template(\"env\", type: :unix)\n  :ok\n  ```\n\n  will generate the file `env` with the following contents:\n\n  ```bash\n  # DOCS My port\n  # TYPE integer\n  export MYAPP_PORT='Elixir.Application'\n  ```\n\n- For Windows:\n\n  ```elixir\n  iex(1)> Myapp.Config.template(\"env.bat\", type: :windows)\n  :ok\n  ```\n\n  will generate the file `env.bat` with the following contents:\n\n  ```bat\n  :: DOCS My port\n  :: TYPE integer\n  SET MYAPP_PORT=\"Elixir.Application\"\n  ```","ref":"readme.html#automatic-template-generation"},{"type":"extras","title":"Custom variable bindings - Skogsrå","doc":"By default, Skogsrå loads variables in the following order:\n\n- OS environment variables or `:system`.\n- Application configuratuon or `:config`.\n\nThis means that every variable has the following defaults:\n\n- For `binding_order` is `[:system, :config]`.\n- For `binding_skip` is `[]`.\n\nThese two values can be modified globally via configuration e.g:\n\n- For globally changing the variable binding order:\n\n   ```elixir\n   config :skogsra,\n     binding_order: [:config, :system]\n   ```\n\n- For globally skipping `:system`:\n\n   ```elixir\n   config :skogsra,\n     binding_skip: [:system]\n   ```\n\nOr be modified per `app_env` e.g:\n\n- For changing `MyApp.Config.my_port/0` binding order:\n\n   ```elixir\n   defmodule MyApp.Config do\n     use Skogsra\n\n     @envdoc \"My port\"\n     app_env :my_port, :myapp, :port,\n       binding_order: [:config, :system]\n   end\n   ```\n\n- For skipping `:system` in `MyApp.Config.my_port/0`:\n\n   ```elixir\n   defmodule MyApp.Config do\n     use Skogsra\n\n     @envdoc \"My port\"\n     app_env :my_port, :myapp, :port,\n       binding_skip: [:system]\n   end\n   ```\n\nAdditionally, we can create new variable bindings by implementing\n`Skogsra.Binding` behaviour e.g. an implementation for loading JSON\nconfiguration files would be:\n\n```elixir\ndefmodule MyApp.Json do\n  use Skogsra.Binding\n\n  alias Skogsra.Env\n\n  @impl true\n  def init(%Env{} = env) do\n    options = Env.extra_options(env)\n\n    case options[:config_path] do\n      nil ->\n        {:error, \"JSON config path not specified\"}\n\n      path ->\n        load(path)\n    end\n  end\n\n  @impl true\n  def get_env(%Env{} = env, config) when is_map(config) do\n    name = Env.os_env(env)\n    value = config[name]\n\n    {:ok, value}\n  end\n\n  # Helpers\n\n  # Loads JSON once and caches it in a :persistent_term using the path\n  # as the key.\n  @spec load(binary()) :: {:ok, map()} | {:error, term()}\n  defp load(path) do\n    with nil  \n        {:error, \"Cannot load #{path} due to #{inspect(reason)}\"}\n\n      config ->\n        {:ok, config}\n    end\n  end\nend\n```\n\nThe previous implementation expects variables to be named the same as the\nOS environment variables e.g:\n\n```json\n{\n  \"MYAPP_PORT\": 5000\n}\n```\n\nThen our variable declaration would look something like the following:\n\n```elixir\ndefmodule MyApp.Config do\n  use Skogsra\n\n  @envdoc \"My port\"\n  app_env :my_port, :myapp, :port,\n    binding_order: [:system, :config, MyApp.Json],\n    config_path: \"#{Path.cwd!()}/priv/config.json\",\n    default: 4000\nend\n```\n\nIf no `:system` or `:config` is found, it will try to load the variable from\nthe JSON file e.g:\n\n```elixir\niex> MyApp.Config.port()\n{:ok, 5000}\n```\n\n> **Note**: The same casting rules apply for all bindings.","ref":"readme.html#custom-variable-bindings"},{"type":"extras","title":"Elixir release YAML and JSON config providers - Skogsrå","doc":"Skogsrå includes two simple configuration providers compatible with\n`mix release` for Elixir ≥ 1.9:\n\n- YAML configuration provider:\n\n   ```yaml\n   - app: \"my_app\"              # Name of the application.\n     module: \"MyApp.Repo\"       # Optional module/namespace.\n     config:                    # Actual configuration.\n       database: \"my_app_db\"\n       username: \"postgres\"\n       password: \"postgres\"\n       hostname: \"localhost\"\n       port: 5432\n   ```\n\n- JSON configuration provider:\n\n   ```json\n   [\n     {\n       \"app\": \"my_app\",\n       \"module\": \"MyApp.Repo\",\n       \"config\": {\n         \"database\": \"my_app_db\",\n         \"username\": \"postgres\",\n         \"password\": \"postgres\",\n         \"hostname\": \"localhost\",\n         \"port\": 5432\n       }\n     }\n   ]\n   ```\n\nBoth configurations would be equivalent to:\n\n```elixir\nconfig :my_app, MyApp.Repo,\n  database: \"my_App_db\",\n  username: \"postgres\"\n  password: \"postgres\"\n  hostname: \"localhost\"\n  port: 5432\n```\n\nFor using these config providers, just add the following to your release\nconfiguration:\n\n- For YAML configurations:\n\n   ```elixir\n   config_providers: [{Skogsra.Provider.Yaml, [\"/path/to/config/file.yml\"]}]\n   ```\n\n- For JSON configurations:\n\n   ```elixir\n   config_providers: [{Skogsra.Provider.Json, [\"/path/to/config/file.json\"]}]\n   ```\n\n> **Note**: If the `module` you're using in you're config does not exist, then\n> change it to `namespace` e.g: `namespace: \"MyApp.Repo\"`. Otherwise, it will\n> fail loading it.","ref":"readme.html#elixir-release-yaml-and-json-config-providers"},{"type":"extras","title":"Using with `Hab` - Skogsrå","doc":"[_Hab_](https://github.com/alexdesousa/hab) is an\n[Oh My ZSH](https://github.com/robbyrussell/oh-my-zsh) plugin for loading OS\nenvironment variables automatically.\n\nBy default, `Hab` will try to load `.envrc` file, but it's possible to have\nseveral of those files for different purposes e.g:\n\n- `.envrc.prod` for production OS variables.\n- `.envrc.test` for testing OS variables.\n- `.envrc` for development variables.\n\n`Hab` will load the development variables by default, but it can load the\nother files using the command `hab_load  ` e.g. loading\n`.envrc.prod` would be as follows:\n\n```bash\n~/my_project $ hab_load prod\n[SUCCESS]  Loaded hab [/home/user/my_project/.envrc.prod]\n```","ref":"readme.html#using-with-hab"},{"type":"extras","title":"Installation - Skogsrå","doc":"The package can be installed by adding `skogsra` to your list of dependencies\nin `mix.exs` (Elixir ≥ 1.11 and Erlang ≥ 22):\n\n```elixir\ndef deps do\n  [\n    {:skogsra, \"~> 2.4\"}\n  ]\nend\n```\n\nIf you need YAML config provider support, add the following:\n\n```elixir\ndef deps do\n  [\n    {:skogsra, \"~> 2.4\"},\n    {:yamerl, \"~> 0.10\"}\n  ]\nend\n```\n\nIf you need JSON config provider support, add  the following:\n\n```elixir\ndef deps do\n  [\n    {:skogsra, \"~> 2.4\"},\n    {:jason, \"~> 1.4\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Author - Skogsrå","doc":"Alexander de Sousa.","ref":"readme.html#author"},{"type":"extras","title":"License - Skogsrå","doc":"_Skogsrå_ is released under the MIT License. See the LICENSE file for further\ndetails.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Changelog for 2.4.2 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-4-2"},{"type":"extras","title":"Bugfix - Changelog","doc":"* Change from `Logger.warn/1` to `Logger.warning/1` to avoid deprecation\n    message on compile time.\n  * Updated dependencies.","ref":"changelog.html#bugfix"},{"type":"extras","title":"Changelog for 2.4.1 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-4-1"},{"type":"extras","title":"Bugfix - Changelog","doc":"* Bugfix: Documentation generation throwed warnings for custom Skogsra types.\n  * Updated dependencies.","ref":"changelog.html#bugfix"},{"type":"extras","title":"Changelog for 2.4.0 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-4-0"},{"type":"extras","title":"Bugfix - Changelog","doc":"* Bugfix: [YAML Provider] fixed bug where it would duplicate apps.\n  * Bugfix: [YAML Provider] fixed bug where regular lists weren't accepted as values.\n  * Migrated pipeline from Travis to Github Actions.\n  * Improved code coverage.\n  * Updated dependencies.","ref":"changelog.html#bugfix"},{"type":"extras","title":"Changelog for 2.3.3 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-3-3"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `:neg_integer`, `:non_neg_integer` and `:pos_integer` as custom\n    types.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 2.3.2 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-3-2"},{"type":"extras","title":"Bugfix - Changelog","doc":"* Wrong spec in private function.","ref":"changelog.html#bugfix"},{"type":"extras","title":"Changelog for 2.3.1 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-3-1"},{"type":"extras","title":"Bugfix - Changelog","doc":"* Multiline docs in templates weren't properly commented.","ref":"changelog.html#bugfix"},{"type":"extras","title":"Changelog for 2.3.0 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `preload/0` and `preload/1` functions for overriding Elixir config\n    values when config providers and OS system variables have precedence.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 2.2.3 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-2-3"},{"type":"extras","title":"Bugfix - Changelog","doc":"* When there is no default and it's not required, a variable should have the\n    variable's type and `nil` as return value e.g. for the variable `:my_var`\n    that returns integer, the spec would be as follows:\n\n    ```elixir\n    @spec my_var() :: {:ok, nil | integer()} | {:error, binary()}\n    ```","ref":"changelog.html#bugfix"},{"type":"extras","title":"Changelog for 2.2.2 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-2-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `validate!/0` and `validate!/1` functions for checking whether the\n    required variables have values or not (thanks to\n    [Riccardo Binetti](https://github.com/rbino)).\n  * Travis-CI build now checking `format`, `credo`, `dialyzer` and `coveralls`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 2.2.1 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-2-1"},{"type":"extras","title":"Bugfix - Changelog","doc":"* When no type is defined and it cannot be derived from the default value,\n    then it should return `any()` instead of `binary()`.","ref":"changelog.html#bugfix"},{"type":"extras","title":"Changelog for 2.2.0 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added JSON config provider.\n  * Added new `Skogsra.Binding` behaviour for adding custom variable bindings.\n  * New option `binding_order` for setting a custom order for variable binding\n    (defaults to `[:system, :config]`). Possible values:\n    + `:system`: For loading OS environment variables.\n    + `:config`: For loading application configuration variables.\n    + `module()`: For loading variables using a custom module implementing\n      `Skogsra.Binding` behaviour.\n  * Added `binding_skip` option to skip one or several variable binding types\n    (defaults to `[]`).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Improved YAML config provider by making it equivalent to the JSON provider.\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Deprecations - Changelog","doc":"* The option `skip_config` was deprecated favoring `binding_skip: [:config]`.\n  * The option `skip_system` was deprecated favoring `binding_skip: [:system]`.","ref":"changelog.html#deprecations"},{"type":"extras","title":"Changelog for 2.1.1 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-1-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fixed bug where `Skogsra.Type` couldn't be used as a type (see [Custom Type Fails at get_spec_type](https://github.com/gmtprime/skogsra/issues/4))","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Changelog for 2.1.0 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Improved function specs.\n  * Improved generated docs.\n  * Added option to avoid automatically generated docs.\n  * Added function for OS environment variable generation for Unix, Releases\n    and Windows.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 2.0.4 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-0-4"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Improved documentation.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 2.0.3 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-0-3"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Improved errors when a variable is missing.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 2.0.2 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-0-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `:module` built-in type.\n  * Added `:unsafe_module` built-in type.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 2.0.0 - Changelog","doc":"#","ref":"changelog.html#changelog-for-2-0-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `Skogsra.Type` behaviour for defining custom types.\n  * Application configuration values are now casted as well to the defined\n    type in the `Skogsra` environment variable definition.\n  * Added YAML configuration provider.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 1.3.0 - Changelog","doc":"#","ref":"changelog.html#changelog-for-1-3-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Namespaces now inherit values from default namespace.\n  * Favoring the use of `:persistent_term`s over `:ets` tables. This avoids the\n    creation of an application tree for this project, thus making it even\n    faster.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changelog for 1.2.1 - Changelog","doc":"#","ref":"changelog.html#changelog-for-1-2-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fixed bug where `Skogsra.Cache` didn't work at compilation time.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Changelog for 1.2.0 - Changelog","doc":"#","ref":"changelog.html#changelog-for-1-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added autogenerated reload functions.\n  * Added autogenerated setter functions.\n  * Improved documentation.\n  * Code rearrangement and refactor for maintainability.","ref":"changelog.html#enhancements"}],"content_type":"text/markdown"}